import os
import unittest
from unittest.mock import patch

from pyobas.configuration.sources import DictionarySource, EnvironmentSource

TEST_ENV_VAR: str = "PYOBAS_TEST_ENV_VAR"


class TestEnvironmentSource(unittest.TestCase):
    @patch.dict(os.environ, clear=True)
    def test_when_env_var_not_set_return_none(self):
        result = EnvironmentSource.get(TEST_ENV_VAR)

        self.assertIsNone(result)

    @patch.dict(os.environ, values={TEST_ENV_VAR: "some_value"}, clear=True)
    def test_when_env_var_is_set_return_value(self):
        result = EnvironmentSource.get(TEST_ENV_VAR)

        self.assertEqual(result, "some_value")


class TestDictionarySource(unittest.TestCase):
    def test_when_config_key_path_is_not_list_it_throws(self):
        string_key_path = "some string"
        with self.assertRaises(AssertionError):
            DictionarySource.get(string_key_path, {})

    def test_when_config_key_path_has_not_2_elements_it_throws(self):
        key_path = ["element 1"]
        with self.assertRaises(AssertionError):
            DictionarySource.get(key_path, {})

    def test_when_config_key_path_has_empty_elements_it_throws(self):
        key_path = ["element 1", ""]
        with self.assertRaises(AssertionError):
            DictionarySource.get(key_path, {})

    def test_when_config_missing_first_path_part_return_None(self):
        expected_value = None
        key_path = ["element 1", "element 2"]
        values = {"not element 1": None}

        result = DictionarySource.get(key_path, values)

        self.assertEqual(result, expected_value)

    def test_when_config_missing_second_path_part_return_None(self):
        expected_value = None
        key_path = ["element 1", "element 2"]
        values = {"element 1": {"not element 2": "some value"}}

        result = DictionarySource.get(key_path, values)

        self.assertEqual(result, expected_value)

    def test_when_config_found_return_value(self):
        expected_value = "expected!"
        key_path = ["element 1", "element 2"]
        values = {"element 1": {"element 2": expected_value}}

        result = DictionarySource.get(key_path, values)

        self.assertEqual(result, expected_value)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import mock_open, patch

from pyobas.configuration import Configuration
from pyobas.daemons import CollectorDaemon
from pyobas.daemons.collector_daemon import DEFAULT_PERIOD_SECONDS


class TestCollectorDaemon(unittest.TestCase):
    @patch("pyobas.apis.DocumentManager.upsert")
    @patch("pyobas.apis.CollectorManager.create")
    @patch("builtins.open", new_callable=mock_open, read_data="data")
    @patch("pyobas.utils.PingAlive.start")
    def test_when_no_period_config_provided_set_default_period(
        self,
        mock_ping_alive,
        mock_open_local,
        mock_collector_create,
        mock_document_upsert,
    ):
        mock_ping_alive.return_value = None
        mock_collector_create.return_value = {}
        mock_document_upsert.return_value = {}
        config = Configuration(
            config_hints={
                "openbas_url": {"data": "fake"},
                "openbas_token": {"data": "fake"},
                "collector_id": {"data": "fake id"},
            }
        )
        collector = CollectorDaemon(config)

        collector._setup()

        self.assertEqual(config.get("collector_period"), DEFAULT_PERIOD_SECONDS)


if __name__ == "__main__":
    unittest.main()



import json
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from enum import Enum
from typing import List

from pyobas import utils
from pyobas.contracts.contract_utils import ContractCardinality, ContractVariable
from pyobas.contracts.variable_helper import VariableHelper


class SupportedLanguage(str, Enum):
    fr: str = "fr"
    en: str = "en"


class ContractFieldType(str, Enum):
    Text: str = "text"
    Number: str = "number"
    Tuple: str = "tuple"
    Checkbox: str = "checkbox"
    Textarea: str = "textarea"
    Select: str = "select"
    Article: str = "article"
    Challenge: str = "challenge"
    DependencySelect: str = "dependency-select"
    Attachment: str = "attachment"
    Team: str = "team"
    Expectation: str = "expectation"
    Asset: str = "asset"
    AssetGroup: str = "asset-group"
    Payload: str = "payload"


class ContractOutputType(str, Enum):
    Text: str = "text"
    Number: str = "number"
    Port: str = "port"
    PortsScan: str = "portscan"
    IPv4: str = "ipv4"
    IPv6: str = "ipv6"
    CVE: str = "cve"


class ExpectationType(str, Enum):
    text: str = "TEXT"
    document: str = "DOCUMENT"
    article: str = "ARTICLE"
    challenge: str = "CHALLENGE"
    manual: str = "MANUAL"
    prevention: str = "PREVENTION"
    detection: str = "DETECTION"
    vulnerability: str = "VULNERABILITY"


@dataclass
class Expectation:
    expectation_type: ExpectationType
    expectation_name: str
    expectation_description: str
    expectation_score: int
    expectation_expectation_group: bool


@dataclass
class LinkedFieldModel:
    key: str
    type: ContractFieldType


@dataclass
class ContractElement(ABC):
    key: str
    label: str
    type: str = field(default="", init=False)
    mandatoryGroups: List[str] = None
    mandatoryConditionField: str = None
    mandatoryConditionValue: str = None
    linkedFields: List["ContractElement"] = field(default_factory=list)
    linkedValues: List[str] = field(default_factory=list)
    mandatory: bool = False
    readOnly: bool = False

    @property
    @abstractmethod
    def get_type(self) -> str:
        pass

    def __post_init__(self):
        self.type = self.get_type


@dataclass
class ContractCardinalityElement(ContractElement, ABC):
    cardinality: str = ContractCardinality.One
    defaultValue: List[str] = field(default_factory=list)


@dataclass
class ContractOutputElement(ABC):
    type: str
    field: str
    labels: List[str]
    isFindingCompatible: bool
    isMultiple: bool


@dataclass
class ContractConfig:
    type: str
    expose: bool
    label: dict[SupportedLanguage, str]
    color_dark: str
    color_light: str


@dataclass
class Contract:
    contract_id: str
    label: dict[SupportedLanguage, str]
    fields: List[ContractElement]
    outputs: List[ContractOutputElement]
    config: ContractConfig
    manual: bool
    variables: List[ContractVariable] = field(
        default_factory=lambda: [
            VariableHelper.user_variable(),
            VariableHelper.exercise_variable(),
            VariableHelper.team_variable(),
        ]
        + VariableHelper.uri_variables()
    )
    contract_attack_patterns_external_ids: List[str] = field(default_factory=list)
    is_atomic_testing: bool = True
    platforms: List[str] = field(default_factory=list)

    def add_attack_pattern(self, var: str):
        self.contract_attack_patterns_external_ids.append(var)

    def add_variable(self, var: ContractVariable):
        self.variables.append(var)


@dataclass
class ContractTeam(ContractCardinalityElement):
    @property
    def get_type(self) -> str:
        return ContractFieldType.Team.value


@dataclass
class ContractText(ContractCardinalityElement):

    defaultValue: str = ""

    @property
    def get_type(self) -> str:
        return ContractFieldType.Text.value


def prepare_contracts(contracts):
    return list(
        map(
            lambda c: {
                "contract_id": c.contract_id,
                "contract_labels": c.label,
                "contract_attack_patterns_external_ids": c.contract_attack_patterns_external_ids,
                "contract_content": json.dumps(c, cls=utils.EnhancedJSONEncoder),
                "contract_platforms": c.platforms,
            },
            contracts,
        )
    )


@dataclass
class ContractTuple(ContractCardinalityElement):
    def __post_init__(self):
        super().__post_init__()
        self.cardinality = ContractCardinality.Multiple

    attachmentKey: str = None
    contractAttachment: bool = attachmentKey is not None
    tupleFilePrefix: str = "file :: "

    @property
    def get_type(self) -> str:
        return ContractFieldType.Tuple.value


@dataclass
class ContractTextArea(ContractCardinalityElement):

    defaultValue: str = ""
    richText: bool = False

    @property
    def get_type(self) -> str:
        return ContractFieldType.Textarea.value


@dataclass
class ContractCheckbox(ContractElement):

    defaultValue: bool = False

    @property
    def get_type(self) -> str:
        return ContractFieldType.Checkbox.value


@dataclass
class ContractAttachment(ContractCardinalityElement):

    @property
    def get_type(self) -> str:
        return ContractFieldType.Attachment.value


@dataclass
class ContractExpectations(ContractCardinalityElement):
    cardinality = ContractCardinality.Multiple
    predefinedExpectations: List[Expectation] = field(default_factory=list)

    @property
    def get_type(self) -> str:
        return ContractFieldType.Expectation.value


@dataclass
class ContractSelect(ContractCardinalityElement):

    choices: dict[str, str] = None

    @property
    def get_type(self) -> str:
        return ContractFieldType.Select.value


@dataclass
class ContractAsset(ContractCardinalityElement):

    @property
    def get_type(self) -> str:
        return ContractFieldType.Asset.value


@dataclass
class ContractAssetGroup(ContractCardinalityElement):

    @property
    def get_type(self) -> str:
        return ContractFieldType.AssetGroup.value


@dataclass
class ContractPayload(ContractCardinalityElement):

    @property
    def get_type(self) -> str:
        return ContractFieldType.Payload.value



import copy
import importlib
import json
import pprint
import textwrap
from types import ModuleType
from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Type, Union

from pyobas.exceptions import OpenBASParsingError

from . import utils
from .client import OpenBAS, OpenBASList

__all__ = [
    "RESTObject",
    "RESTObjectList",
    "RESTManager",
]


class RESTObject:
    _id_attr: Optional[str] = "id"
    _attrs: Dict[str, Any]
    _created_from_list: bool  # Indicates if object was created from a list() action
    _module: ModuleType
    _parent_attrs: Dict[str, Any]
    _repr_attr: Optional[str] = None
    _updated_attrs: Dict[str, Any]
    manager: "RESTManager"

    def __init__(
        self,
        manager: "RESTManager",
        attrs: Dict[str, Any],
        *,
        created_from_list: bool = False,
    ) -> None:
        if not isinstance(attrs, dict):
            raise OpenBASParsingError(
                f"Attempted to initialize RESTObject with a non-dictionary value: "
                f"{attrs!r}\nThis likely indicates an incorrect or malformed server "
                f"response."
            )
        self.__dict__.update(
            {
                "manager": manager,
                "_attrs": attrs,
                "_updated_attrs": {},
                "_module": importlib.import_module(self.__module__),
                "_created_from_list": created_from_list,
            }
        )
        self.__dict__["_parent_attrs"] = self.manager.parent_attrs
        self._create_managers()

    def __getstate__(self) -> Dict[str, Any]:
        state = self.__dict__.copy()
        module = state.pop("_module")
        state["_module_name"] = module.__name__
        return state

    def __setstate__(self, state: Dict[str, Any]) -> None:
        module_name = state.pop("_module_name")
        self.__dict__.update(state)
        self.__dict__["_module"] = importlib.import_module(module_name)

    def __getattr__(self, name: str) -> Any:
        if name in self.__dict__["_updated_attrs"]:
            return self.__dict__["_updated_attrs"][name]

        if name in self.__dict__["_attrs"]:
            value = self.__dict__["_attrs"][name]
            if isinstance(value, list):
                self.__dict__["_updated_attrs"][name] = value[:]
                return self.__dict__["_updated_attrs"][name]

            return value

        if name in self.__dict__["_parent_attrs"]:
            return self.__dict__["_parent_attrs"][name]

        message = f"{type(self).__name__!r} object has no attribute {name!r}"
        if self._created_from_list:
            message = f"{message}\n\n" + textwrap.fill(
                f"{self.__class__!r} was created via a list() call and "
                f"only a subset of the data may be present. To ensure "
                f"all data is present get the object using a "
                f"get(object.id) call. For more details, see:"
            )
        raise AttributeError(message)

    def __setattr__(self, name: str, value: Any) -> None:
        self.__dict__["_updated_attrs"][name] = value

    def asdict(self, *, with_parent_attrs: bool = False) -> Dict[str, Any]:
        data = {}
        if with_parent_attrs:
            data.update(copy.deepcopy(self._parent_attrs))
        data.update(copy.deepcopy(self._attrs))
        data.update(copy.deepcopy(self._updated_attrs))
        return data

    @property
    def attributes(self) -> Dict[str, Any]:
        return self.asdict(with_parent_attrs=True)

    def to_json(self, *, with_parent_attrs: bool = False, **kwargs: Any) -> str:
        return json.dumps(self.asdict(with_parent_attrs=with_parent_attrs), **kwargs)

    def __str__(self) -> str:
        return f"{type(self)} => {self.asdict()}"

    def pformat(self) -> str:
        return f"{type(self)} => \n{pprint.pformat(self.asdict())}"

    def pprint(self) -> None:
        print(self.pformat())

    def __repr__(self) -> str:
        name = self.__class__.__name__

        if (self._id_attr and self._repr_value) and (self._id_attr != self._repr_attr):
            return (
                f"<{name} {self._id_attr}:{self.get_id()} "
                f"{self._repr_attr}:{self._repr_value}>"
            )
        if self._id_attr:
            return f"<{name} {self._id_attr}:{self.get_id()}>"
        if self._repr_value:
            return f"<{name} {self._repr_attr}:{self._repr_value}>"

        return f"<{name}>"

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, RESTObject):
            return NotImplemented
        if self.get_id() and other.get_id():
            return self.get_id() == other.get_id()
        return super() == other

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, RESTObject):
            return NotImplemented
        if self.get_id() and other.get_id():
            return self.get_id() != other.get_id()
        return super() != other

    def __dir__(self) -> Iterable[str]:
        return set(self.attributes).union(super().__dir__())

    def __hash__(self) -> int:
        if not self.get_id():
            return super().__hash__()
        return hash(self.get_id())

    def _create_managers(self) -> None:
        # NOTE(jlvillal): We are creating our managers by looking at the class
        # annotations. If an attribute is annotated as being a *Manager type
        # then we create the manager and assign it to the attribute.
        for attr, annotation in sorted(self.__annotations__.items()):
            # We ignore creating a manager for the 'manager' attribute as that
            # is done in the self.__init__() method
            if attr in ("manager",):
                continue
            if not isinstance(annotation, (type, str)):
                continue
            if isinstance(annotation, type):
                cls_name = annotation.__name__
            else:
                cls_name = annotation
            # All *Manager classes are used except for the base "RESTManager" class
            if cls_name == "RESTManager" or not cls_name.endswith("Manager"):
                continue
            cls = getattr(self._module, cls_name)
            manager = cls(self.manager.openbas, parent=self)
            # Since we have our own __setattr__ method, we can't use setattr()
            self.__dict__[attr] = manager

    def _update_attrs(self, new_attrs: Dict[str, Any]) -> None:
        self.__dict__["_updated_attrs"] = {}
        self.__dict__["_attrs"] = new_attrs

    def get_id(self) -> Optional[Union[int, str]]:
        """Returns the id of the resource."""
        if self._id_attr is None or not hasattr(self, self._id_attr):
            return None
        id_val = getattr(self, self._id_attr)
        if TYPE_CHECKING:
            assert id_val is None or isinstance(id_val, (int, str))
        return id_val

    @property
    def _repr_value(self) -> Optional[str]:
        """Safely returns the human-readable resource name if present."""
        if self._repr_attr is None or not hasattr(self, self._repr_attr):
            return None
        repr_val = getattr(self, self._repr_attr)
        if TYPE_CHECKING:
            assert isinstance(repr_val, str)
        return repr_val

    @property
    def encoded_id(self) -> Optional[Union[int, str]]:
        """Ensure that the ID is url-encoded so that it can be safely used in a URL
        path"""
        obj_id = self.get_id()
        if isinstance(obj_id, str):
            obj_id = utils.EncodedId(obj_id)
        return obj_id


class RESTObjectList:
    def __init__(
        self, manager: "RESTManager", obj_cls: Type[RESTObject], _list: OpenBASList
    ) -> None:
        self.manager = manager
        self._obj_cls = obj_cls
        self._list = _list

    def __iter__(self) -> "RESTObjectList":
        return self

    def __len__(self) -> int:
        return len(self._list)

    def __next__(self) -> RESTObject:
        return self.next()

    def next(self) -> RESTObject:
        data = self._list.next()
        return self._obj_cls(self.manager, data, created_from_list=True)

    @property
    def current_page(self) -> int:
        """The current page number."""
        return self._list.current_page

    @property
    def prev_page(self) -> Optional[int]:
        """The previous page number.

        If None, the current page is the first.
        """
        return self._list.prev_page

    @property
    def next_page(self) -> Optional[int]:
        """The next page number.

        If None, the current page is the last.
        """
        return self._list.next_page

    @property
    def per_page(self) -> Optional[int]:
        """The number of items per page."""
        return self._list.per_page

    @property
    def total_pages(self) -> Optional[int]:
        """The total number of pages."""
        return self._list.total_pages

    @property
    def total(self) -> Optional[int]:
        """The total number of items."""
        return self._list.total


class RESTManager:
    """Base class for CRUD operations on objects.

    Derived class must define ``_path`` and ``_obj_cls``.

    ``_path``: Base URL path on which requests will be sent (e.g. '/projects')
    ``_obj_cls``: The class of objects that will be created
    """

    _create_attrs: Any
    _update_attrs: Any
    _path: Optional[str] = None
    _obj_cls: Optional[Type[RESTObject]] = None
    _from_parent_attrs: Dict[str, Any] = {}
    _types: Dict[str, Any] = {}

    _computed_path: Optional[str]
    _parent: Optional[RESTObject]
    _parent_attrs: Dict[str, Any]
    openbas: OpenBAS

    def __init__(self, openbas: OpenBAS, parent: Optional[RESTObject] = None) -> None:
        self.openbas = openbas
        self._parent = parent  # for nested managers
        self._computed_path = self._compute_path()

    @property
    def parent_attrs(self) -> Optional[Dict[str, Any]]:
        return self._parent_attrs

    def _compute_path(self, path: Optional[str] = None) -> Optional[str]:
        self._parent_attrs = {}
        if path is None:
            path = self._path
        if path is None:
            return None
        if self._parent is None or not self._from_parent_attrs:
            return path

        data: Dict[str, Optional[utils.EncodedId]] = {}
        for self_attr, parent_attr in self._from_parent_attrs.items():
            if not hasattr(self._parent, parent_attr):
                data[self_attr] = None
                continue
            data[self_attr] = utils.EncodedId(getattr(self._parent, parent_attr))
        self._parent_attrs = data
        return path.format(**data)

    @property
    def path(self) -> Optional[str]:
        return self._computed_path



from typing import Any, Dict, List

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject
from pyobas.mixins import CreateMixin
from pyobas.utils import RequiredOptional


class InjectExpectationTrace(RESTObject):
    _id_attr = "inject_expectation_trace_id"


class InjectExpectationTraceManager(CreateMixin, RESTManager):
    _path = "/inject-expectations-traces"
    _obj_cls = InjectExpectationTrace
    _create_attrs = RequiredOptional(
        required=(
            "inject_expectation_trace_expectation",
            "inject_expectation_trace_source_id",
            "inject_expectation_trace_alert_name",
            "inject_expectation_trace_alert_link",
            "inject_expectation_trace_date",
        ),
    )

    @exc.on_http_error(exc.OpenBASUpdateError)
    def bulk_create(
        self, payload: Dict[str, List[Dict[str, str]]], **kwargs: Any
    ) -> dict[str, Any]:
        path = f"{self.path}/bulk"
        result = self.openbas.http_post(
            path,
            post_data=payload,
            **kwargs,
        )
        return result



import unittest

from pyobas.signatures.signature_type import SignatureType
from pyobas.signatures.types import MatchTypes, SignatureTypes


class TestSignatureType(unittest.TestCase):
    def test_make_struct_create_expected_struct_for_simple_sig_type(self):
        simple_signature_type_label = SignatureTypes.SIG_TYPE_HOSTNAME
        simple_signature_type = SignatureType(
            label=simple_signature_type_label, match_type=MatchTypes.MATCH_TYPE_SIMPLE
        )

        data = "just a simple string"
        simple_struct = simple_signature_type.make_struct_for_matching(data=data)

        self.assertEqual(simple_struct.get("type"), MatchTypes.MATCH_TYPE_SIMPLE.value)
        self.assertEqual(simple_struct.get("data"), data)
        self.assertFalse("score" in simple_struct.keys())

    def test_make_struct_create_expected_struct_for_fuzzy_sig_type(self):
        fuzzy_signature_type_label = SignatureTypes.SIG_TYPE_HOSTNAME
        fuzzy_signature_type_score = 50
        fuzzy_signature_type = SignatureType(
            label=fuzzy_signature_type_label,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=fuzzy_signature_type_score,
        )

        data = "just another simple string"
        simple_struct = fuzzy_signature_type.make_struct_for_matching(data=data)

        self.assertEqual(simple_struct.get("type"), MatchTypes.MATCH_TYPE_FUZZY.value)
        self.assertEqual(simple_struct.get("data"), data)
        self.assertEqual(simple_struct.get("score"), fuzzy_signature_type_score)

    def test_make_struct_create_expected_struct_for_fuzzy_sig_type_when_score_is_0(
        self,
    ):
        fuzzy_signature_type_label = SignatureTypes.SIG_TYPE_HOSTNAME
        fuzzy_signature_type_score = 0
        fuzzy_signature_type = SignatureType(
            label=fuzzy_signature_type_label,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=fuzzy_signature_type_score,
        )

        data = "just another simple string"
        simple_struct = fuzzy_signature_type.make_struct_for_matching(data=data)

        self.assertEqual(simple_struct.get("type"), MatchTypes.MATCH_TYPE_FUZZY.value)
        self.assertEqual(simple_struct.get("data"), data)
        self.assertEqual(simple_struct.get("score"), fuzzy_signature_type_score)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from unittest.mock import patch

from pyobas.configuration import Configuration

TEST_CONFIG_HINTS = {
    "string_config_direct": {"data": "this is string_config_direct"},
    "string_config_no_default": {
        "env": "PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_NO_DEFAULT",
        "file_path": ["pyobas_test_configuration", "string_config_no_default"],
    },
    "int_config_no_default": {
        "env": "PYOBAS_TEST_CONFIGURATION_INT_CONFIG_NO_DEFAULT",
        "file_path": ["pyobas_test_configuration", "int_config_no_default"],
        "is_number": True,
    },
    "int_config_no_default_not_marked_number": {
        "env": "PYOBAS_TEST_CONFIGURATION_INT_CONFIG_NO_DEFAULT",
        "file_path": [
            "pyobas_test_configuration",
            "int_config_no_default_not_marked_number",
        ],
    },
    "string_config_with_default": {
        "env": "PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_WITH_DEFAULT",
        "file_path": ["pyobas_test_configuration", "string_config_with_default"],
        "default": "default string config",
    },
    "int_config_with_default": {
        "env": "PYOBAS_TEST_CONFIGURATION_INT_CONFIG_WITH_DEFAULT",
        "file_path": ["pyobas_test_configuration", "int_config_with_default"],
        "is_number": True,
        "default": 777_777,
    },
    "bool_config_with_default": {
        "env": "PYOBAS_TEST_CONFIGURATION_BOOL_CONFIG_WITH_DEFAULT",
        "file_path": ["pyobas_test_configuration", "bool_config_with_default"],
        "default": True,
    },
}


class TestConfiguration(unittest.TestCase):
    def test_when_string_config_has_no_default_when_key_not_found_return_None(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("string_config_no_default")

        self.assertIsNone(value)

    def test_when_config_has_no_default_when_key_is_set_as_bool_return_bool(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )
        config_obj.set("string_config_no_default", True)

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, True)

    def test_when_config_has_no_default_when_key_is_set_as_string_bool_return_bool(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )
        config_obj.set("string_config_no_default", "yes")

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, True)

    def test_when_string_config_has_default_when_key_not_found_return_default(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("string_config_with_default")

        self.assertEqual(value, "default string config")

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_NO_DEFAULT": "actual value"},
        clear=True,
    )
    def test_when_string_config_has_no_default_when_key_is_in_env_return_env_value(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, "actual value")

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_WITH_DEFAULT": "actual value"},
        clear=True,
    )
    def test_when_string_config_has_default_when_key_is_in_env_return_env_value(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("string_config_with_default")

        self.assertEqual(value, "actual value")

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_NO_DEFAULT": "actual value"},
        clear=True,
    )
    def test_when_key_is_in_both_env_and_file_return_env_value(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
            config_values={
                "pyobas_test_configuration": {
                    "string_config_no_default": "another value"
                }
            },
        )

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, "actual value")

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_NO_DEFAULT": "env value"},
        clear=True,
    )
    def test_when_key_is_in_both_env_and_file_when_value_is_set_return_set_value(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
            config_values={
                "pyobas_test_configuration": {"string_config_no_default": "file value"}
            },
        )
        config_obj.set("string_config_no_default", "set value")

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, "set value")

    def test_when_string_config_has_no_default_when_key_is_in_file_return_file_value(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
            config_values={
                "pyobas_test_configuration": {
                    "string_config_no_default": "another value"
                }
            },
        )

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, "another value")

    def test_when_int_config_has_no_default_when_key_is_not_found_return_None(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("int_config_no_default")

        self.assertIsNone(value)

    def test_when_int_config_has_default_when_key_is_not_found_return_default(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("int_config_with_default")

        self.assertEqual(value, 777_777)

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_INT_CONFIG_NO_DEFAULT": "1234"},
        clear=True,
    )
    def test_when_int_config_has_no_default_when_key_is_in_env_return_env_value(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("int_config_no_default")

        self.assertEqual(value, 1234)

    def test_when_int_config_has_no_default_when_key_is_passed_as_int_return_int_value(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
            config_values={
                "pyobas_test_configuration": {"int_config_no_default": 456_123}
            },
        )

        value = config_obj.get("int_config_no_default")

        self.assertEqual(value, 456_123)

    def test_when_int_config_has_no_default_when_key_is_passed_as_int_when_config_is_not_marked_as_number_return_int_value(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
            config_values={
                "pyobas_test_configuration": {
                    "int_config_no_default_not_marked_number": 842_204
                }
            },
        )

        value = config_obj.get("int_config_no_default_not_marked_number")

        self.assertEqual(value, 842_204)

    @patch.dict(
        os.environ,
        values={"PYOBAS_TEST_CONFIGURATION_STRING_CONFIG_NO_DEFAULT": "yes"},
        clear=True,
    )
    def test_when_string_config_has_no_default_when_string_is_boolean_when_key_is_in_env_return_env_value(
        self,
    ):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("string_config_no_default")

        self.assertEqual(value, True)

    def test_when_bool_config_has_default_when_key_is_not_found_return_default(self):
        config_obj = Configuration(
            config_hints=TEST_CONFIG_HINTS,
        )

        value = config_obj.get("bool_config_with_default")

        self.assertEqual(value, True)


if __name__ == "__main__":
    unittest.main()



from typing import Any, Dict

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class Payload(RESTObject):
    _id_attr = "payload_id"


class PayloadManager(RESTManager):
    _path = "/payloads"
    _obj_cls = Payload

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(self, payload: Dict[str, Any], **kwargs: Any) -> Dict[str, Any]:
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(path, post_data=payload, **kwargs)
        return result

    @exc.on_http_error(exc.OpenBASUpdateError)
    def deprecate(
        self, payloads_processed: Dict[str, Any], **kwargs: Any
    ) -> Dict[str, Any]:
        path = f"{self.path}/deprecate"
        result = self.openbas.http_post(path, post_data=payloads_processed, **kwargs)
        return result



from typing import TYPE_CHECKING, Any, BinaryIO, Dict, List, Optional, Union
from urllib import parse

import requests

from pyobas import exceptions, utils
from pyobas._version import __version__  # noqa: F401

REDIRECT_MSG = (
    "pyobas detected a {status_code} ({reason!r}) redirection. You must update "
    "your OpenBVAS URL to the correct URL to avoid issues. The redirection was from: "
    "{source!r} to {target!r}"
)


class OpenBAS:
    def __init__(
        self,
        url: str,
        token: str,
        timeout: Optional[float] = None,
        per_page: Optional[int] = None,
        pagination: Optional[str] = None,
        order_by: Optional[str] = None,
        ssl_verify: Union[bool, str] = True,
        **kwargs: Any,
    ) -> None:

        if url is None or len(url) == 0:
            raise ValueError("An URL must be set")
        if token is None or len(token) == 0 or token == "ChangeMe":
            raise ValueError("A TOKEN must be set")

        self.url = url
        self.timeout = timeout
        #: Headers that will be used in request to OpenBAS
        self.headers = {
            "User-Agent": "pyobas/" + __version__,
            "Authorization": "Bearer " + token,
        }
        #: Whether SSL certificates should be validated
        self.ssl_verify = ssl_verify

        # Import backends
        from pyobas import backends

        self.backend = backends.RequestsBackend(**kwargs)
        self._auth = backends.TokenAuth(token)
        self.session = self.backend.client

        self.per_page = per_page
        self.pagination = pagination
        self.order_by = order_by

        # Import all apis
        from pyobas import apis

        self.me = apis.MeManager(self)
        self.organization = apis.OrganizationManager(self)
        self.injector = apis.InjectorManager(self)
        self.collector = apis.CollectorManager(self)
        self.inject = apis.InjectManager(self)
        self.document = apis.DocumentManager(self)
        self.kill_chain_phase = apis.KillChainPhaseManager(self)
        self.attack_pattern = apis.AttackPatternManager(self)
        self.team = apis.TeamManager(self)
        self.endpoint = apis.EndpointManager(self)
        self.user = apis.UserManager(self)
        self.inject_expectation = apis.InjectExpectationManager(self)
        self.payload = apis.PayloadManager(self)
        self.security_platform = apis.SecurityPlatformManager(self)
        self.inject_expectation_trace = apis.InjectExpectationTraceManager(self)
        self.tag = apis.TagManager(self)

    @staticmethod
    def _check_redirects(result: requests.Response) -> None:
        # Check the requests history to detect 301/302 redirections.
        # If the initial verb is POST or PUT, the redirected request will use a
        # GET request, leading to unwanted behaviour.
        # If we detect a redirection with a POST or a PUT request, we
        # raise an exception with a useful error message.
        if not result.history:
            return

        for item in result.history:
            if item.status_code not in (301, 302):
                continue
            # GET methods can be redirected without issue
            if item.request.method == "GET":
                continue
            target = item.headers.get("location")
            raise exceptions.RedirectError(
                REDIRECT_MSG.format(
                    status_code=item.status_code,
                    reason=item.reason,
                    source=item.url,
                    target=target,
                )
            )

    def _build_url(self, path: str) -> str:
        """Returns the full url from path.

        If path is already a url, return it unchanged. If it's a path, append
        it to the stored url.

        Returns:
            The full URL
        """
        if path.startswith("http://") or path.startswith("https://"):
            return path
        return f"{self.url}/api{path}"

    def _get_session_opts(self) -> Dict[str, Any]:
        return {
            "headers": self.headers.copy(),
            "auth": self._auth,
            "timeout": self.timeout,
            "verify": self.ssl_verify,
        }

    def http_request(
        self,
        verb: str,
        path: str,
        query_data: Optional[Dict[str, Any]] = None,
        post_data: Optional[Union[Dict[str, Any], bytes, BinaryIO]] = None,
        raw: bool = False,
        streamed: bool = False,
        files: Optional[Dict[str, Any]] = None,
        timeout: Optional[float] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """Make an HTTP request to the OpenBAS server.

        Args:
            verb: The HTTP method to call ('get', 'post', 'put', 'delete')
            path: Path or full URL to query ('/projects' or
                        'http://whatever/v4/api/projecs')
            query_data: Data to send as query parameters
            post_data: Data to send in the body (will be converted to
                              json by default)
            raw: If True, do not convert post_data to json
            streamed: Whether the data should be streamed
            files: The files to send to the server
            timeout: The timeout, in seconds, for the request
            **kwargs: Extra options to send to the server (e.g. sudo)

        Returns:
            A requests result object.

        Raises:
            OpenBASHttpError: When the return code is not 2xx
        """
        query_data = query_data or {}
        raw_url = self._build_url(path)

        # parse user-provided URL params to ensure we don't add our own duplicates
        parsed = parse.urlparse(raw_url)
        params = parse.parse_qs(parsed.query)
        utils.copy_dict(src=query_data, dest=params)

        url = parse.urlunparse(parsed._replace(query=""))

        if "query_parameters" in kwargs:
            utils.copy_dict(src=kwargs["query_parameters"], dest=params)
            for arg in ("per_page", "page"):
                if arg in kwargs:
                    params[arg] = kwargs[arg]
        else:
            utils.copy_dict(src=kwargs, dest=params)

        opts = self._get_session_opts()

        verify = opts.pop("verify")
        opts_timeout = opts.pop("timeout")
        # If timeout was passed into kwargs, allow it to override the default
        if timeout is None:
            timeout = opts_timeout

        # We need to deal with json vs. data when uploading files
        send_data = self.backend.prepare_send_data(files, post_data, raw)
        opts["headers"]["Content-type"] = send_data.content_type

        # cur_retries = 0
        while True:
            # noinspection PyTypeChecker
            result = self.backend.http_request(
                method=verb,
                url=url,
                json=send_data.json,
                data=send_data.data,
                params=params,
                timeout=timeout,
                verify=verify,
                stream=streamed,
                **opts,
            )

            self._check_redirects(result.response)

            if 200 <= result.status_code < 300:
                return result.response

            error_message = result.content
            try:
                error_json = result.json()
                for k in ("message", "error"):
                    if k in error_json:
                        error_message = error_json[k]
            except (KeyError, ValueError, TypeError):
                pass

            if result.status_code == 401:
                raise exceptions.OpenBASAuthenticationError(
                    response_code=result.status_code,
                    error_message=error_message,
                    response_body=result.content,
                )

            raise exceptions.OpenBASHttpError(
                response_code=result.status_code,
                error_message=error_message,
                response_body=result.content,
            )

    def http_get(
        self,
        path: str,
        query_data: Optional[Dict[str, Any]] = None,
        streamed: bool = False,
        raw: bool = False,
        **kwargs: Any,
    ) -> Union[Dict[str, Any], requests.Response]:
        query_data = query_data or {}
        result = self.http_request(
            "get", path, query_data=query_data, streamed=streamed, **kwargs
        )
        content_type = utils.get_content_type(result.headers.get("Content-Type"))

        if content_type == "application/json" and not streamed and not raw:
            try:
                json_result = result.json()
                return json_result
            except Exception as e:
                raise exceptions.OpenBASParsingError(
                    error_message="Failed to parse the server message"
                ) from e
        else:
            return result

    def http_head(
        self, path: str, query_data: Optional[Dict[str, Any]] = None, **kwargs: Any
    ) -> "requests.structures.CaseInsensitiveDict[Any]":
        query_data = query_data or {}
        result = self.http_request("head", path, query_data=query_data, **kwargs)
        return result.headers

    def http_post(
        self,
        path: str,
        query_data: Optional[Dict[str, Any]] = None,
        post_data: Optional[Dict[str, Any]] = None,
        raw: bool = False,
        files: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> Union[Dict[str, Any], requests.Response]:
        query_data = query_data or {}
        post_data = post_data or {}
        result = self.http_request(
            "post",
            path,
            query_data=query_data,
            post_data=post_data,
            files=files,
            raw=raw,
            **kwargs,
        )
        content_type = utils.get_content_type(result.headers.get("Content-Type"))

        try:
            if content_type == "application/json":
                json_result = result.json()
                return json_result
        except Exception as e:
            raise exceptions.OpenBASParsingError(
                error_message="Failed to parse the server message"
            ) from e
        return result

    def http_put(
        self,
        path: str,
        query_data: Optional[Dict[str, Any]] = None,
        post_data: Optional[Union[Dict[str, Any], bytes, BinaryIO]] = None,
        raw: bool = False,
        files: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> Union[Dict[str, Any], requests.Response]:
        query_data = query_data or {}
        post_data = post_data or {}
        result = self.http_request(
            "put",
            path,
            query_data=query_data,
            post_data=post_data,
            files=files,
            raw=raw,
            **kwargs,
        )
        try:
            json_result = result.json()
            if TYPE_CHECKING:
                assert isinstance(json_result, dict)
            return json_result
        except Exception as e:
            raise exceptions.OpenBASParsingError(
                error_message="Failed to parse the server message"
            ) from e

    def http_patch(
        self,
        path: str,
        *,
        query_data: Optional[Dict[str, Any]] = None,
        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
        raw: bool = False,
        **kwargs: Any,
    ) -> Union[Dict[str, Any], requests.Response]:
        query_data = query_data or {}
        post_data = post_data or {}

        result = self.http_request(
            "patch",
            path,
            query_data=query_data,
            post_data=post_data,
            raw=raw,
            **kwargs,
        )
        try:
            json_result = result.json()
            if TYPE_CHECKING:
                assert isinstance(json_result, dict)
            return json_result
        except Exception as e:
            raise exceptions.OpenBASParsingError(
                error_message="Failed to parse the server message"
            ) from e

    def http_delete(self, path: str, **kwargs: Any) -> requests.Response:
        return self.http_request("delete", path, **kwargs)

    def http_list(
        self,
        path: str,
        query_data: Optional[Dict[str, Any]] = None,
        *,
        iterator: Optional[bool] = None,
        **kwargs: Any,
    ) -> Union["OpenBASList", List[Dict[str, Any]]]:
        query_data = query_data or {}

        url = self._build_url(path)

        page = kwargs.get("page")

        if iterator and page is None:
            # Generator requested
            return OpenBASList(self, url, query_data, **kwargs)

        # pagination requested, we return a list
        bas_list = OpenBASList(self, url, query_data, get_next=False, **kwargs)
        items = list(bas_list)
        return items


class OpenBASList:
    """Generator representing a list of remote objects.

    The object handles the links returned by a query to the API, and will call
    the API again when needed.
    """

    def __init__(
        self,
        openbas: OpenBAS,
        url: str,
        query_data: Dict[str, Any],
        get_next: bool = True,
        **kwargs: Any,
    ) -> None:
        self._openbas = openbas

        # Preserve kwargs for subsequent queries
        self._kwargs = kwargs.copy()

        self._query(url, query_data, **self._kwargs)
        self._get_next = get_next

        # Remove query_parameters from kwargs, which are saved via the `next` URL
        self._kwargs.pop("query_parameters", None)

    def _query(
        self, url: str, query_data: Optional[Dict[str, Any]] = None, **kwargs: Any
    ) -> None:
        query_data = query_data or {}
        result = self._openbas.http_request("get", url, query_data=query_data, **kwargs)
        try:
            next_url = result.links["next"]["url"]
        except KeyError:
            next_url = None

        self._next_url = next_url
        self._current_page: Optional[str] = result.headers.get("X-Page")
        self._prev_page: Optional[str] = result.headers.get("X-Prev-Page")
        self._next_page: Optional[str] = result.headers.get("X-Next-Page")
        self._per_page: Optional[str] = result.headers.get("X-Per-Page")
        self._total_pages: Optional[str] = result.headers.get("X-Total-Pages")
        self._total: Optional[str] = result.headers.get("X-Total")

        try:
            self._data: List[Dict[str, Any]] = result.json()
        except Exception as e:
            raise exceptions.OpenBASParsingError(
                error_message="Failed to parse the server message"
            ) from e

        self._current = 0

    @property
    def current_page(self) -> int:
        """The current page number."""
        if TYPE_CHECKING:
            assert self._current_page is not None
        return int(self._current_page)

    @property
    def prev_page(self) -> Optional[int]:
        """The previous page number.

        If None, the current page is the first.
        """
        return int(self._prev_page) if self._prev_page else None

    @property
    def next_page(self) -> Optional[int]:
        """The next page number.

        If None, the current page is the last.
        """
        return int(self._next_page) if self._next_page else None

    @property
    def per_page(self) -> Optional[int]:
        """The number of items per page."""
        return int(self._per_page) if self._per_page is not None else None

    @property
    def total_pages(self) -> Optional[int]:
        """The total number of pages."""
        if self._total_pages is not None:
            return int(self._total_pages)
        return None

    @property
    def total(self) -> Optional[int]:
        """The total number of items."""
        if self._total is not None:
            return int(self._total)
        return None

    def __iter__(self) -> "OpenBASList":
        return self

    def __len__(self) -> int:
        if self._total is None:
            return 0
        return int(self._total)

    def __next__(self) -> Dict[str, Any]:
        return self.next()

    def next(self) -> Dict[str, Any]:
        try:
            item = self._data[self._current]
            self._current += 1
            return item
        except IndexError:
            pass

        if self._next_url and self._get_next is True:
            self._query(self._next_url, **self._kwargs)
            return self.next()

        raise StopIteration



from typing import Any, Dict

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class Inject(RESTObject):
    _id_attr = None


class InjectManager(RESTManager):
    _path = "/injects"
    _obj_cls = Inject

    @exc.on_http_error(exc.OpenBASUpdateError)
    def execution_callback(
        self, inject_id: str, data: Dict[str, Any], **kwargs: Any
    ) -> Dict[str, Any]:
        path = f"{self.path}/execution/callback/{inject_id}"
        result = self.openbas.http_post(path, post_data=data, **kwargs)
        return result

    @exc.on_http_error(exc.OpenBASUpdateError)
    def execution_reception(
        self, inject_id: str, data: Dict[str, Any], **kwargs: Any
    ) -> Dict[str, Any]:
        path = f"{self.path}/execution/reception/{inject_id}"
        result = self.openbas.http_post(path, post_data=data, **kwargs)
        return result



from pyobas.exceptions import OpenBASError
from pyobas.signatures.types import MatchTypes


class SignatureMatch:
    def __init__(self, match_type: MatchTypes, match_score: int | None):
        if match_score is None and match_type != MatchTypes.MATCH_TYPE_SIMPLE:
            raise OpenBASError(
                f"Match type {match_type} requires score to be set, found score = {match_score}"
            )
        self.match_type = match_type
        self.match_score = match_score



"""
Defines http backends for processing http requests
"""

from .backend import RequestsBackend, RequestsResponse, TokenAuth

DefaultBackend = RequestsBackend
DefaultResponse = RequestsResponse

__all__ = [
    "DefaultBackend",
    "DefaultResponse",
    "TokenAuth",
]



from typing import Any, Dict, List

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class AttackPattern(RESTObject):
    _id_attr = "attack_pattern_id"


class AttackPatternManager(RESTManager):
    _path = "/attack_patterns"
    _obj_cls = AttackPattern

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(
        self,
        attack_patterns: List[Dict[str, Any]],
        ignore_dependencies: bool = False,
        **kwargs: Any,
    ) -> Dict[str, Any]:
        data = {
            "attack_patterns": attack_patterns,
            "ignore_dependencies": ignore_dependencies,
        }
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(path, post_data=data, **kwargs)
        return result



from pyobas.contracts.contract_utils import (
    ContractCardinality,
    ContractVariable,
    VariableType,
)

USER = "user"
EXERCISE = "exercise"
TEAMS = "teams"
COMCHECK = "comcheck"
PLAYER_URI = "player_uri"
CHALLENGES_URI = "challenges_uri"
SCOREBOARD_URI = "scoreboard_uri"
LESSONS_URI = "lessons_uri"


class VariableHelper:
    @staticmethod
    def user_variable():
        return ContractVariable(
            key=USER,
            label="User that will receive the injection",
            type=VariableType.String,
            cardinality=ContractCardinality.One,
            children=[
                ContractVariable(
                    key=USER + ".id",
                    label="Id of the user in the platform",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=USER + ".email",
                    label="Email of the user",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=USER + ".firstname",
                    label="Firstname of the user",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=USER + ".lastname",
                    label="Lastname of the user",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=USER + ".lang",
                    label="Lang of the user",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
            ],
        )

    @staticmethod
    def exercise_variable():
        return ContractVariable(
            key=EXERCISE,
            label="Exercise of the current injection",
            type=VariableType.Object,
            cardinality=ContractCardinality.One,
            children=[
                ContractVariable(
                    key=EXERCISE + ".id",
                    label="Id of the exercise in the platform",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=EXERCISE + ".name",
                    label="Name of the exercise",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
                ContractVariable(
                    key=EXERCISE + ".description",
                    label="Description of the exercise",
                    type=VariableType.String,
                    cardinality=ContractCardinality.One,
                ),
            ],
        )

    @staticmethod
    def team_variable():
        return ContractVariable(
            key=TEAMS,
            label="List of team name for the injection",
            type=VariableType.String,
            cardinality=ContractCardinality.Multiple,
        )

    @staticmethod
    def uri_variables():
        return [
            ContractVariable(
                key=PLAYER_URI,
                label="Player interface platform link",
                type=VariableType.String,
                cardinality=ContractCardinality.One,
            ),
            ContractVariable(
                key=CHALLENGES_URI,
                label="Challenges interface platform link",
                type=VariableType.String,
                cardinality=ContractCardinality.One,
            ),
            ContractVariable(
                key=SCOREBOARD_URI,
                label="Scoreboard interface platform link",
                type=VariableType.String,
                cardinality=ContractCardinality.One,
            ),
            ContractVariable(
                key=LESSONS_URI,
                label="Lessons learned interface platform link",
                type=VariableType.String,
                cardinality=ContractCardinality.One,
            ),
        ]



from typing import Any, Dict, List

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class KillChainPhase(RESTObject):
    _id_attr = "phase_id"


class KillChainPhaseManager(RESTManager):
    _path = "/kill_chain_phases"
    _obj_cls = KillChainPhase

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(
        self, kill_chain_phases: List[Dict[str, Any]], **kwargs: Any
    ) -> Dict[str, Any]:
        data = {"kill_chain_phases": kill_chain_phases}
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(path, post_data=data, **kwargs)
        return result



import unittest
import unittest.mock

from pyobas.configuration import Configuration
from pyobas.daemons import BaseDaemon
from pyobas.exceptions import OpenBASError

TEST_DAEMON_CONFIG_HINTS = {
    "openbas_url": {"data": "http://example.com"},
    "openbas_token": {"data": "test"},
    "log_level": {"data": "info"},
    "name": {"data": "my test daemon"},
}

TEST_DAEMON_CONFIGURATION = Configuration(config_hints=TEST_DAEMON_CONFIG_HINTS)


class DaemonForTest(BaseDaemon):
    def _setup(self):
        pass

    def _start_loop(self):
        pass


def create_mock_daemon(callback: callable = None):
    mock_setup = unittest.mock.MagicMock()
    mock_start_loop = unittest.mock.MagicMock()
    inner_mock_func = unittest.mock.MagicMock()

    class MockDaemon(BaseDaemon):
        def _setup(self):
            mock_setup()

        def _start_loop(self):
            mock_start_loop()

        def bound_method(self):
            inner_mock_func()

    return (
        MockDaemon(
            configuration=TEST_DAEMON_CONFIGURATION,
            callback=callback,
            # silence logging in tests
            logger=unittest.mock.MagicMock(),
        ),
        mock_setup,
        mock_start_loop,
        inner_mock_func,
    )


class TestBaseDaemon(unittest.TestCase):
    def test_when_no_callback_when_complete_config_ctor_ok(self):
        daemon = DaemonForTest(configuration=TEST_DAEMON_CONFIGURATION)

        self.assertIsInstance(daemon, BaseDaemon)

    def test_when_no_callback_when_lacking_config_key_ctor_throws(self):
        with self.assertRaises(Exception):
            DaemonForTest(configuration=Configuration(config_hints={}))

    def test_when_no_callback_daemon_cant_start(self):
        daemon, mock_setup, mock_start_loop, _ = create_mock_daemon()

        with self.assertRaises(OpenBASError):
            daemon.start()

        mock_setup.assert_not_called()
        mock_start_loop.assert_not_called()

    def test_when_callback_daemon_can_start(self):
        daemon, mock_setup, mock_start_loop, _ = create_mock_daemon(lambda: None)

        daemon.start()

        mock_setup.assert_called_once()
        mock_start_loop.assert_called_once()

    def test_when_callback_is_bound_method_daemon_can_call(self):
        daemon, mock_setup, mock_start_loop, inner_mock_func = create_mock_daemon()
        daemon.set_callback(daemon.bound_method)

        daemon._try_callback()

        inner_mock_func.assert_called_once()

    def test_when_callback_is_func_with_collector_parameter_daemon_can_call(self):
        daemon, mock_setup, mock_start_loop, _ = create_mock_daemon()
        inner_mock_func = unittest.mock.MagicMock()
        daemon.set_callback(lambda collector: inner_mock_func())

        daemon._try_callback()

        inner_mock_func.assert_called_once()

    def test_when_callback_is_func_with_other_parameter_daemon_cant_call(self):
        daemon, mock_setup, mock_start_loop, _ = create_mock_daemon()
        inner_mock_func = unittest.mock.MagicMock()
        daemon.set_callback(lambda other_parameter: inner_mock_func())

        daemon._try_callback()

        inner_mock_func.assert_not_called()


if __name__ == "__main__":
    unittest.main()



import argparse
import logging
import os

import requests
from OBAS_utils.release_utils import check_release, closeRelease

logging.basicConfig(encoding="utf-8", level=logging.INFO)

parser = argparse.ArgumentParser("release")
parser.add_argument(
    "branch_client_python", help="The new version number of the release.", type=str
)
parser.add_argument(
    "previous_version", help="The previous version number of the release.", type=str
)
parser.add_argument(
    "new_version", help="The new version number of the release.", type=str
)
parser.add_argument(
    "github_token", help="The github token to use for the release note.", type=str
)
parser.add_argument(
    "--dev", help="Flag to prevent pushing the release.", action="store_true"
)
args = parser.parse_args()

previous_version = args.previous_version
new_version = args.new_version
branch_client_python = args.branch_client_python
github_token = args.github_token

os.environ["DRONE_COMMIT_AUTHOR"] = "Filigran-Automation"
os.environ["GIT_AUTHOR_NAME"] = "Filigran Automation"
os.environ["GIT_AUTHOR_EMAIL"] = "automation@filigran.io"
os.environ["GIT_COMMITTER_NAME"] = "Filigran Automation"
os.environ["GIT_COMMITTER_EMAIL"] = "automation@filigran.io"

# Python library release
logging.info("[client-python] Starting the release")
with open("./pyobas/__init__.py", "r") as file:
    filedata = file.read()
filedata = filedata.replace(previous_version, new_version)
with open("./pyobas/__init__.py", "w") as file:
    file.write(filedata)
with open("./pyobas/_version.py", "r") as file:
    filedata = file.read()
filedata = filedata.replace(previous_version, new_version)
with open("./pyobas/_version.py", "w") as file:
    file.write(filedata)

# Commit the change
logging.info("[client-python] Pushing to " + branch_client_python)
os.system('git commit -a -m "[client] Release ' + new_version + '"')
if not args.dev:
    os.system("git push origin " + branch_client_python)

logging.info("[client-python] Tagging")
os.system("git tag -f " + new_version)
if not args.dev:
    os.system("git push -f --tags > /dev/null 2>&1")

logging.info("[client-python] Generating release")
os.system("gren release > /dev/null 2>&1")

# Modify the release note
logging.info("[client-python] Getting the current release note")
release = requests.get(
    "https://api.github.com/repos/OpenBAS-Platform/client-python/releases/latest",
    headers={
        "Accept": "application/vnd.github+json",
        "Authorization": "Bearer " + github_token,
        "X-GitHub-Api-Version": "2022-11-28",
    },
)
release_data = release.json()
release_body = release_data["body"]

logging.info("[client-python] Generating the new release note")
if not args.dev:
    github_release_note = requests.post(
        "https://api.github.com/repos/OpenBAS-Platform/client-python/releases/generate-notes",
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": "Bearer " + github_token,
            "X-GitHub-Api-Version": "2022-11-28",
        },
        json={"tag_name": new_version, "previous_tag_name": previous_version},
    )
    github_release_note_data = github_release_note.json()
    github_release_note_data_body = github_release_note_data["body"]
    if "Full Changelog" not in release_body:
        new_release_note = (
            release_body
            + "\n"
            + github_release_note_data_body.replace(
                "## What's Changed", "#### Pull Requests:\n"
            ).replace("## New Contributors", "#### New Contributors:\n")
        )
    else:
        new_release_note = release_body

    logging.info("[client-python] Updating the release")
    requests.patch(
        "https://api.github.com/repos/OpenBAS-Platform/client-python/releases/"
        + str(release_data["id"]),
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": "Bearer " + github_token,
            "X-GitHub-Api-Version": "2022-11-28",
        },
        json={"body": new_release_note},
    )

if not args.dev:
    closeRelease(
        "https://api.github.com/repos/OpenBAS-Platform/client-python",
        new_version,
        github_token,
    )

logging.info(
    "[client-python] Release done! Waiting 10 minutes for CI/CD and publication..."
)

if not args.dev:
    check_release("https://pypi.org/simple/pyobas/", "pyobas-" + new_version, 10)



import unittest.mock
from uuid import uuid4

from pyobas.apis.inject_expectation.model import (
    DetectionExpectation,
    PreventionExpectation,
)
from pyobas.signatures.signature_type import SignatureType
from pyobas.signatures.types import MatchTypes, SignatureTypes


def create_mock_api_client():
    return unittest.mock.MagicMock()


class TestExpectation(unittest.TestCase):
    def test_when_detection_model_when_success_update_labels_correct(self):
        inject_expectation_id = uuid4()
        sender_id = uuid4()
        api_client = create_mock_api_client()
        model = DetectionExpectation(
            **{
                "inject_expectation_id": inject_expectation_id,
                "inject_expectation_signatures": [],
            },
            api_client=api_client,
        )

        model.update(success=True, sender_id=sender_id, metadata={})

        api_client.update.assert_called_once_with(
            inject_expectation_id,
            inject_expectation={
                "collector_id": sender_id,
                "result": "Detected",
                "is_success": True,
                "metadata": {},
            },
        )

    def test_when_detection_model_when_failure_update_labels_correct(self):
        inject_expectation_id = uuid4()
        sender_id = uuid4()
        api_client = create_mock_api_client()
        model = DetectionExpectation(
            **{
                "inject_expectation_id": inject_expectation_id,
                "inject_expectation_signatures": [],
            },
            api_client=api_client,
        )  # we don't care to construct a functional object

        model.update(success=False, sender_id=sender_id, metadata={})

        api_client.update.assert_called_once_with(
            inject_expectation_id,
            inject_expectation={
                "collector_id": sender_id,
                "result": "Not Detected",
                "is_success": False,
                "metadata": {},
            },
        )

    def test_when_prevention_model_when_success_update_labels_correct(self):
        inject_expectation_id = uuid4()
        sender_id = uuid4()
        api_client = create_mock_api_client()
        model = PreventionExpectation(
            **{
                "inject_expectation_id": inject_expectation_id,
                "inject_expectation_signatures": [],
            },
            api_client=api_client,
        )  # we don't care to construct a functional object

        model.update(success=True, sender_id=sender_id, metadata={})

        api_client.update.assert_called_once_with(
            inject_expectation_id,
            inject_expectation={
                "collector_id": sender_id,
                "result": "Prevented",
                "is_success": True,
                "metadata": {},
            },
        )

    def test_when_prevention_model_when_failure_update_labels_correct(self):
        inject_expectation_id = uuid4()
        sender_id = uuid4()
        api_client = create_mock_api_client()
        model = PreventionExpectation(
            **{
                "inject_expectation_id": inject_expectation_id,
                "inject_expectation_signatures": [],
            },
            api_client=api_client,
        )  # we don't care to construct a functional object

        model.update(success=False, sender_id=sender_id, metadata={})

        api_client.update.assert_called_once_with(
            inject_expectation_id,
            inject_expectation={
                "collector_id": sender_id,
                "result": "Not Prevented",
                "is_success": False,
                "metadata": {},
            },
        )

    def test_when_no_expectation_signature_is_relevant_match_alert_return_false(self):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                ],
            },
            api_client=create_mock_api_client(),
        )

        relevant_signature_types = [
            SignatureType(
                label=SignatureTypes.SIG_TYPE_HOSTNAME,
                match_type=MatchTypes.MATCH_TYPE_SIMPLE,
            )
        ]

        alert_data = (
            {  # irrelevant but aligned with expectation sigs to validate no match
                SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME.value: {
                    "type": MatchTypes.MATCH_TYPE_FUZZY.value,
                    "data": "parent.exe",
                    "score": 95,
                },
            }
        )

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertFalse(matched)

    def test_when_relevant_signature_when_none_match_alert_return_false(self):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                ],
            },
            api_client=create_mock_api_client(),
        )

        parent_process_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        relevant_signature_types = [parent_process_signature_type]

        alert_data = {
            parent_process_signature_type.label.value: parent_process_signature_type.make_struct_for_matching(
                data="not_parent.exe"
            )
        }

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertFalse(matched)

    def test_when_relevant_signature_when_all_signatures_match_alert_return_true(self):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                ],
            },
            api_client=create_mock_api_client(),
        )

        parent_process_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        relevant_signature_types = [parent_process_signature_type]

        alert_data = {
            parent_process_signature_type.label.value: parent_process_signature_type.make_struct_for_matching(
                data="parent.exe"
            )
        }

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertTrue(matched)

    def test_when_relevant_signature_when_all_signatures_match_alert_when_passing_array_return_true(
        self,
    ):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                ],
            },
            api_client=create_mock_api_client(),
        )

        parent_process_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        relevant_signature_types = [parent_process_signature_type]

        alert_data = {
            parent_process_signature_type.label.value: parent_process_signature_type.make_struct_for_matching(
                data=["parent.exe", "some_other_process"]
            )
        }

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertTrue(matched)

    def test_when_relevant_signatures_when_alert_data_missing_for_some_relevant_signatures_return_false(
        self,
    ):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                    {"type": SignatureTypes.SIG_TYPE_FILE_NAME, "value": "filename"},
                ],
            },
            api_client=create_mock_api_client(),
        )

        parent_process_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        file_name_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_FILE_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        relevant_signature_types = [
            parent_process_signature_type,
            file_name_signature_type,
        ]

        alert_data = {
            parent_process_signature_type.label.value: parent_process_signature_type.make_struct_for_matching(
                data="parent.exe"
            )
        }

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertFalse(matched)

    def test_when_relevant_signatures_when_some_alert_data_dont_match_return_false(
        self,
    ):
        model = DetectionExpectation(
            **{
                "inject_expectation_id": uuid4(),
                "inject_expectation_signatures": [
                    {
                        "type": SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
                        "value": "parent.exe",
                    },
                    {
                        "type": SignatureTypes.SIG_TYPE_FILE_NAME,
                        "value": "some_file.odt",
                    },
                ],
            },
            api_client=create_mock_api_client(),
        )

        parent_process_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_PARENT_PROCESS_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        file_name_signature_type = SignatureType(
            label=SignatureTypes.SIG_TYPE_FILE_NAME,
            match_type=MatchTypes.MATCH_TYPE_FUZZY,
            match_score=95,
        )
        relevant_signature_types = [
            parent_process_signature_type,
            file_name_signature_type,
        ]

        alert_data = {
            parent_process_signature_type.label.value: parent_process_signature_type.make_struct_for_matching(
                data="parent.exe"
            ),
            file_name_signature_type.label.value: file_name_signature_type.make_struct_for_matching(
                data="some_other_file.doc"
            ),
        }

        matched = model.match_alert(relevant_signature_types, alert_data)

        self.assertFalse(matched)


if __name__ == "__main__":
    unittest.main()



from typing import Any, Dict

from pyobas import exceptions as exc
from pyobas.base import RESTManager, RESTObject


class Tag(RESTObject):
    _id_attr = "tag_id"


class TagManager(RESTManager):
    _path = "/tags"
    _obj_cls = Tag

    @exc.on_http_error(exc.OpenBASUpdateError)
    def upsert(self, data: Dict[str, Any], **kwargs: Any) -> Dict[str, Any]:
        path = f"{self.path}/upsert"
        result = self.openbas.http_post(path, post_data=data, **kwargs)
        return result



from typing import List

from pyobas.contracts.contract_config import ContractElement, ContractOutputElement


class ContractBuilder:
    fields: List[ContractElement]
    outputs: List[ContractOutputElement]

    def __init__(self):
        self.fields = []
        self.outputs = []

    def add_fields(self, fields: List[ContractElement]):
        self.fields = self.fields + fields
        return self

    def add_outputs(self, outputs: List[ContractOutputElement]):
        self.outputs = self.outputs + outputs
        return self

    def mandatory(self, element: ContractElement):
        element.mandatory = True
        self.fields.append(element)
        return self

    def optional(self, element: ContractElement):
        element.mandatory = False
        self.fields.append(element)
        return self

    def mandatory_group(self, elements: List[ContractElement]):
        keys: List[str] = list(map(lambda iterable: iterable.key, elements))
        for element in elements:
            element.mandatory = True
            element.mandatoryGroups = keys
            self.fields.append(element)
        return self

    def build_fields(self) -> List[ContractElement]:
        return self.fields

    def build_outputs(self) -> List[ContractOutputElement]:
        return self.outputs



# -*- coding: utf-8 -*-
__version__ = "1.18.6"

from pyobas._version import (  # noqa: F401
    __author__,
    __copyright__,
    __email__,
    __license__,
    __title__,
)
from pyobas.client import OpenBAS  # noqa: F401
from pyobas.configuration import *  # noqa: F401,F403,F405
from pyobas.contracts import *  # noqa: F401,F403,F405
from pyobas.exceptions import *  # noqa: F401,F403,F405
from pyobas.signatures import *  # noqa: F401,F403,F405

__all__ = [
    "__author__",
    "__copyright__",
    "__email__",
    "__license__",
    "__title__",
    "__version__",
    "OpenBAS",
]
__all__.extend(exceptions.__all__)  # noqa: F405



import unittest

from pyobas.exceptions import OpenBASError
from pyobas.signatures.signature_match import SignatureMatch
from pyobas.signatures.types import MatchTypes


class TestSignatureMatch(unittest.TestCase):
    def test_non_simple_match_with_non_null_score_throws(self):
        score = None
        self.assertRaises(
            OpenBASError, SignatureMatch, MatchTypes.MATCH_TYPE_FUZZY, score
        )

    def test_simple_match_with_null_score_does_not_throw(self):
        score = None
        SignatureMatch(match_type=MatchTypes.MATCH_TYPE_SIMPLE, match_score=score)

    def test_fuzzy_match_with_0_score_does_not_throw(self):
        score = 0
        SignatureMatch(match_type=MatchTypes.MATCH_TYPE_FUZZY, match_score=score)


if __name__ == "__main__":
    unittest.main()
